#!/usr/bin/env python

from .hw_vendors import HW_vendor
import itertools
import logging

logger = logging.getLogger(__name__)

class Parser:
    MARKER_START_FRAME = chr(2)
    MARKER_STOP_FRAME = chr(3)
    MARKER_END_LINE = '\r\n'

    def __init__(self, hw=None):
        assert hw is not None and isinstance(hw, HW_vendor)
        self._hw = hw
        self._synchro_debut_trame()

    def __iter__(self):
        while True:
            yield self.get_frame()

    def get_frame(self):
        raw = self._get_raw_frame().strip(self.MARKER_END_LINE)
        try:
            groups = [line.split(" ", 2) for line in raw.split(self.MARKER_END_LINE)]
            frame = dict([
                (k, v) for k, v, chksum in groups if chksum == self._checksum(k, v)
            ])
            if len(frame) != len(groups):
                logger.info("Discarded fields because of bad checksum: {}".format(
                    [f for f in itertools.ifilterfalse(lambda g: g[2] == self._checksum(g[0], g[1]), groups)]
                ))
        except Exception as e:
            logger.error("Caught exception while parsing teleinfo frame: {}".format(e))
            frame = {}
        return frame

    def _synchro_debut_trame(self):
        while self._hw.read_char() != self.MARKER_START_FRAME:
            pass

    def _get_raw_frame(self):
        self._synchro_debut_trame()
        frame = ''.join(itertools.takewhile(
            lambda c: c != self.MARKER_STOP_FRAME,
            self._hw)
        )
        return frame

    def _checksum(self, key, value):
        chksum = 32
        chksum += sum([ord(c) for c in key])
        chksum += sum([ord(c) for c in value])
        chksum = (chksum & 63) + 32
        return chr(chksum)




#!/usr/bin/env python

import serial
import time
from abc import ABCMeta, abstractmethod

class HW_vendor():
    __metaclass__ = ABCMeta

    @abstractmethod
    def read_char(self):
        pass

    def __iter__(self):
        while True:
            yield self.read_char()

class HW_serial_based(HW_vendor):
    __metaclass__ = ABCMeta

    BAUDRATE = 1200
    PARITY = serial.PARITY_EVEN
    STOP_BITS = serial.STOPBITS_ONE
    BYTE_SIZE = serial.SEVENBITS

    def __init__(self, port):
        self._serial_port = serial.Serial(
            port=port,
            baudrate=self.BAUDRATE,
            parity=self.PARITY,
            stopbits=self.STOP_BITS,
            bytesize=self.BYTE_SIZE)

class RpiDom(HW_serial_based):
    CHANNEL_TELEINFO1 = 'A'
    CHANNEL_TELEINFO2 = 'B'

    def __init__(self, port='/dev/ttyAMA0', *args, **kwargs):
        super(RpiDom, self).__init__(port, *args, **kwargs)
        self.select_channel(self.CHANNEL_TELEINFO1)

    def read_char(self):
        return self._serial_port.read(1)

    def select_channel(self, channel):
        assert channel in [self.CHANNEL_TELEINFO1, self.CHANNEL_TELEINFO2]
        self._serial_port.write(channel)
        time.sleep(1)

class SolarBox_USB(HW_serial_based):
    def __init__(self, port="/dev/ttyUSB0", *args, **kwargs):
        super(SolarBox_USB, self).__init__(port, *args, **kwargs)
        raise NotImplementedError()

class UTInfo2(HW_serial_based):
    def __init__(self, port="/dev/serial/by-id/usb-FTDI_FT230X_Basic_UART_TINFO-1631-if00-port0", *args, **kwargs):
        super(UTInfo2, self).__init__(port, *args, **kwargs)

    def read_char(self):
        return self._serial_port.read(1)





#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys

from setuptools import setup
from setuptools.command.install import install

VERSION = "1.1.1"

def readme():
    """print long description"""
    with open('README.md') as f:
        return f.read()


class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version"""
    description = 'verify that the git tag matches our version'

    def run(self):
        tag = os.getenv('CIRCLE_TAG')

        if tag != VERSION:
            info = "Git tag: {0} does not match the version of this app: {1}".format(
                tag, VERSION
            )
            sys.exit(info)

setup(
    name='teleinfo',
    version=VERSION,
    url='http://github.com/demikl/python-teleinfo',
    author='Mickael Le Baillif',
    author_email='mickael.le.baillif@gmail.com',
    license='MIT',
    description='EDF Teleinfo frame acquisition',
    long_description=readme(),
    long_description_content_type='text/markdown',
    packages=['teleinfo'],
    install_requires=['pyserial'],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Topic :: Home Automation",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ],
    entry_points= {
        'console_scripts': [
            'teleinfo_json=teleinfo.utils:frame_to_json'
        ]
    },
    cmdclass={
        'verify': VerifyVersionCommand,
    }
)



#!/usr/bin/env python

def frame_to_json():
    import logging
    logging.basicConfig()

    import sys
    import teleinfo
    from teleinfo.hw_vendors import RpiDom, SolarBox_USB, UTInfo2
    import getopt
    import json

    ps_name = sys.argv[0]
    argv = sys.argv[1:]

    SUPPORTED_DEVICES = {
        "RpiDom": RpiDom,
        "SolarBox_USB": SolarBox_USB,
        "UTInfo2": UTInfo2
    }

    def usage():
        print("{} -d|--device <hw_device>".format(ps_name))
        print("{} [-h|--help]".format(ps_name))
        print("")
        print("Supported hardware devices are:")
        print("  {}".format(SUPPORTED_DEVICES.keys()))

    device = None
    try:
        opts, args = getopt.getopt(argv, "hd:", ["help", "device="])
    except getopt.GetoptError:
        usage()
        sys.exit(2)
    for opt, arg in opts:
        if opt in ("-h", "--help"):
            usage()
            sys.exit()
        elif opt in ("-d", "--device"):
            try:
                device = SUPPORTED_DEVICES.get(arg)
            except KeyError:
                print("Invalid device: {}".format(arg))
                usage()
                sys.exit(2)
    if device is None:
      print("Missing device argument")
      usage()
      sys.exit(2)

    ti = teleinfo.Parser(device())
    print(json.dumps(ti.get_frame(), indent=2, separators=(',', ':')))
